@@ -394,6 +394,11 @@ parse_packet(u_char *info, const struct pcap_pkthdr *header, const u_char *packe
             if (pkt_hep3) {
                 packet_destroy(pkt);
                 pkt = pkt_hep3;
+                // Replace fake HEP generated frames with captured ones
+                vector_set_destroyer(pkt->frames, vector_generic_destroyer);
+                vector_destroy(pkt->frames);
+                pkt->frames = vector_create(1, 1);
+                packet_add_frame(pkt, header, packet);
             } else {
                 // Complete packet with Transport information
                 packet_set_type(pkt, PACKET_SIP_UDP);
@@ -735,6 +740,7 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
                             frame->data + link_hl + ip_hl + sizeof (struct ip6_frag),
                             ntohs(frame_ip6->ip6_ctlun.ip6_un1.ip6_un1_plen));
                     pkt->proto = frame_ip6f->ip6f_nxt;
+		    len_data-=sizeof(struct ip6_frag);
                 }
                     break;
 #endif
