@@ -139,6 +139,10 @@ struct capture_config {
     struct bpf_program fp;
     //! libpcap dump file handler
     pcap_dumper_t *pd;
+    //! libpcap dump file name
+    const char *dumpfilename;
+    //! inode of the dump file we have open
+    ino_t dump_inode;
     //! Capture sources
     vector_t *sources;
     //! Capture Lock. Avoid parsing and handling data at the same time
@@ -154,6 +158,8 @@ struct capture_info
 {
     //! Flag to determine if capture is running
     bool running;
+    //! Flag to determine if this capture is libpcap
+    bool ispcap;
     //! libpcap link type
     int link;
     //! libpcap link header size
@@ -172,6 +178,8 @@ struct capture_info
     vector_t *ip_reasm;
     //! Packets pending TCP reassembly
     vector_t *tcp_reasm;
+    //! Capture thread function
+    void *(*capture_fn)(void *data);
     //! Capture thread for online capturing
     pthread_t capture_t;
 };
@@ -197,12 +205,11 @@ capture_deinit();
  * @brief Online capture function
  *
  * @param device Device to start capture from
- * @param outfile Dumpfile for captured packets
  *
  * @return 0 on spawn success, 1 otherwise
  */
 int
-capture_online(const char *dev, const char *outfile);
+capture_online(const char *dev);
 
 /**
  * @brief Read from pcap file and fill sngrep sctuctures
@@ -215,7 +222,7 @@ capture_online(const char *dev, const char *outfile);
  * @return 0 if load has been successfull, 1 otherwise
  */
 int
-capture_offline(const char *infile, const char *outfile);
+capture_offline(const char *infile);
 
 /**
  * @brief Read the next package and parse SIP messages
@@ -314,7 +321,7 @@ capture_launch_thread();
  * This function is used as worker thread for capturing filtered packets and
  * pass them to the UI layer.
  */
-void
+void *
 capture_thread(void *none);
 
 /**
@@ -462,6 +469,18 @@ capture_packet_time_sorter(vector_t *vector, void *item);
 void
 capture_close();
 
+/**
+ * @brief Set general capture dumper
+ */
+void
+capture_set_dumper(pcap_dumper_t *dumper, ino_t dump_inode);
+
+/**
+ * @brief Store a packet in dumper file
+ */
+void
+capture_dump_packet(packet_t *packet);
+
 /**
  * @brief Get datalink header size
  *
@@ -473,7 +492,7 @@ datalink_size(int datalink);
  * @brief Open a new dumper file for capture handler
  */
 pcap_dumper_t *
-dump_open(const char *dumpfile);
+dump_open(const char *dumpfile, ino_t* dump_inode);
 
 /**
  * @brief Store a packet in dump file
