@@ -177,6 +177,10 @@ accept_eep_client(void *info)
             if (capture_packet_parse(pkt) != 0) {
                 packet_destroy(pkt);
             }
+
+            // Store this packets in output file
+            capture_dump_packet(pkt);
+
             capture_unlock();
         }
     }
@@ -272,7 +276,9 @@ capture_eep_build_frame_data(
     };
 
     // Allocate memory for payload contents
-    *frame_payload = sng_malloc(sizeof(ether_hdr) + sizeof(ip_hdr) + sizeof(udp_hdr) + payload_size);
+    if (!(*frame_payload = sng_malloc(sizeof(ether_hdr) + sizeof(ip_hdr) + sizeof(udp_hdr) + payload_size))) {
+        return frame_pcap_header;
+    }
 
     // Append all headers to frame contents
     memcpy(*frame_payload + frame_size, (void*) &ether_hdr, sizeof(ether_hdr));
@@ -399,7 +405,8 @@ capture_eep_send_v3(packet_t *pkt)
     unsigned char *data = packet_payload(pkt);
     uint32_t len = packet_payloadlen(pkt);
 
-    hg = sng_malloc(sizeof(struct hep_generic));
+    if (!(hg = sng_malloc(sizeof(struct hep_generic))))
+        return 1;
 
     /* header set "HEP3" */
     memcpy(hg->header.id, "\x48\x45\x50\x33", 4);
@@ -644,6 +651,12 @@ capture_eep_receive_v2()
         pos += sizeof(struct hep_ip6hdr);
     }
 #endif
+    else {
+        // if we don't recognise the address family, then we don't know how long
+        // the IP header is, so we don't know where the time header will start,
+        // so we can't handle the packet
+        return NULL;
+    }
 
     /* PORTS */
     src.port = ntohs(hdr.hp_sport);
@@ -660,13 +673,20 @@ capture_eep_receive_v2()
 
     // Calculate payload size (Total size - headers size)
     header.caplen = header.len = ntohs(hdr.hp_l) - pos;
+    if (pos+header.caplen >= MAX_CAPTURE_LEN)
+        return NULL;
 
     // Copy packet payload
-    payload = sng_malloc(header.caplen + 1);
+    if (!(payload = sng_malloc(header.caplen + 1)))
+        return NULL;
     memcpy(payload, (void*) buffer + pos, header.caplen);
 
     // Build a custom frame pcap header
     frame_pcap_header = capture_eep_build_frame_data(header, payload,header.caplen, src, dst, &frame_payload);
+    if (!frame_payload) {
+        sng_free(payload);
+        return NULL;
+    }
 
     // Create a new packet
     pkt = packet_create((family == AF_INET) ? 4 : 6, proto, src, dst, 0);
@@ -678,9 +698,6 @@ capture_eep_receive_v2()
     // We don't longer require frame payload anymore, because adding the frame to packet clones its memory
     sng_free(frame_payload);
 
-    // Store this packets in output file
-    capture_dump_packet(pkt);
-
     /* FREE */
     sng_free(payload);
     return pkt;
@@ -708,7 +725,7 @@ capture_eep_receive_v3(const u_char *pkt, uint32_t size)
 #endif
     hep_chunk_t payload_chunk;
     hep_chunk_t authkey_chunk;
-    char password[100];
+    char *password;
     int password_len;
     unsigned char *payload = 0;
     uint32_t total_len, pos;
@@ -736,7 +753,6 @@ capture_eep_receive_v3(const u_char *pkt, uint32_t size)
 
     // Initialize structs
     memset(&hg, 0, sizeof(hep_generic_t));
-    memset(&password, 0, sizeof(password));
     memset(&src, 0, sizeof(address_t));
     memset(&dst, 0, sizeof(address_t));
     memset(&header, 0, sizeof(struct pcap_pkthdr));
@@ -749,6 +765,8 @@ capture_eep_receive_v3(const u_char *pkt, uint32_t size)
         return NULL;
 
     total_len = ntohs(hg.header.length);
+    if (total_len >= MAX_CAPTURE_LEN)
+        return NULL;
     pos = sizeof(hep_ctrl_t);
 
     while (pos < total_len) {
@@ -771,6 +789,8 @@ capture_eep_receive_v3(const u_char *pkt, uint32_t size)
 
         switch (chunk_type) {
             case CAPTURE_EEP_CHUNK_INVALID:
+                if (payload)
+                    sng_free(payload);
                 return NULL;
             case CAPTURE_EEP_CHUNK_FAMILY:
                 memcpy(&hg.ip_family, (void*) buffer + pos, sizeof(hep_chunk_uint8_t));
@@ -823,12 +843,16 @@ capture_eep_receive_v3(const u_char *pkt, uint32_t size)
             case CAPTURE_EEP_CHUNK_AUTH_KEY:
                 memcpy(&authkey_chunk, (void*) buffer + pos, sizeof(authkey_chunk));
                 password_len = ntohs(authkey_chunk.length) - sizeof(authkey_chunk);
+                password = sng_malloc(password_len);
                 memcpy(password, (void*) buffer + pos + sizeof(hep_chunk_t), password_len);
                 break;
             case CAPTURE_EEP_CHUNK_PAYLOAD:
                 memcpy(&payload_chunk, (void*) buffer + pos, sizeof(payload_chunk));
                 header.caplen = header.len = chunk_len - sizeof(hep_chunk_t);
-                payload = sng_malloc(header.caplen);
+                if (pos+sizeof(hep_chunk_t)+header.caplen >= MAX_CAPTURE_LEN)
+                    return NULL;
+                if (!(payload = sng_malloc(header.caplen)))
+                    return NULL;
                 memcpy(payload, (void*) buffer + pos + sizeof(hep_chunk_t), header.caplen);
                 break;
             case CAPTURE_EEP_CHUNK_CORRELATION_ID:
@@ -847,12 +871,16 @@ capture_eep_receive_v3(const u_char *pkt, uint32_t size)
         if (strlen(password) == 0)
             return NULL;
         // Check password matches configured
-        if (strncmp(password, eep_cfg.capt_srv_password, strlen(eep_cfg.capt_srv_password)) != 0)
+        if (strcmp(password, eep_cfg.capt_srv_password) != 0)
             return NULL;
     }
 
     // Build a custom frame pcap header
     frame_pcap_header = capture_eep_build_frame_data(header, payload,header.caplen, src, dst, &frame_payload);
+    if (!frame_payload) {
+        sng_free(payload);
+        return NULL;
+    }
 
     // Create a new packet
     pkt_new = packet_create((hg.ip_family.data == AF_INET)?4:6, hg.ip_proto.data, src, dst, 0);
@@ -863,11 +891,10 @@ capture_eep_receive_v3(const u_char *pkt, uint32_t size)
     // We don't longer require frame payload anymore, because adding the frame to packet clones its memory
     sng_free(frame_payload);
 
-    // Store this packets in output file
-    capture_dump_packet(pkt_new);
-
     /* FREE */
     sng_free(payload);
+    sng_free(password);
+
     return pkt_new;
 }
 
@@ -880,7 +907,7 @@ capture_eep_set_server_url(const char *url)
     memset(address, 0, sizeof(address));
     memset(port, 0, sizeof(port));
 
-    strncpy(urlstr, url, sizeof(urlstr));
+    sng_strncpy(urlstr, url, sizeof(urlstr));
     if (sscanf(urlstr, "%*[^:]:%" STRINGIFY(ADDRESSLEN) "[^:]:%5s", address, port) == 2) {
         setting_set_value(SETTING_EEP_LISTEN, SETTING_ON);
         setting_set_value(SETTING_EEP_LISTEN_ADDR, address);
@@ -899,7 +926,7 @@ capture_eep_set_client_url(const char *url)
     memset(address, 0, sizeof(address));
     memset(port, 0, sizeof(port));
 
-    strncpy(urlstr, url, sizeof(urlstr));
+    sng_strncpy(urlstr, url, sizeof(urlstr));
     if (sscanf(urlstr, "%*[^:]:%" STRINGIFY(ADDRESSLEN) "[^:]:%5s", address, port) == 2) {
         setting_set_value(SETTING_EEP_SEND, SETTING_ON);
         setting_set_value(SETTING_EEP_SEND_ADDR, address);
