@@ -577,9 +577,6 @@ call_list_handle_key(ui_t *ui, int key)
     if (info->menu_active)
         return call_list_handle_menu_key(ui, key);
 
-    // Get window of call list panel
-    WINDOW *list_win = info->list_win;
-
     // Check actions for this key
     while ((action = key_find_action(key, action)) != ERR) {
         // Check if we handle this action
@@ -824,7 +821,7 @@ call_list_handle_form_key(ui_t *ui, int key)
     int field_len = strlen(field_buffer(info->fields[FLD_LIST_FILTER], 0));
     dfilter = malloc(field_len + 1);
     memset(dfilter, 0, field_len + 1);
-    strncpy(dfilter, field_buffer(info->fields[FLD_LIST_FILTER], 0), field_len);
+    sng_strncpy(dfilter, field_buffer(info->fields[FLD_LIST_FILTER], 0), field_len);
     // Trim any trailing spaces
     strtrim(dfilter);
 
