@@ -142,7 +142,7 @@ call_raw_print_msg(ui_t *ui, sip_msg_t *msg)
     getmaxyx(pad, height, width);
 
     // Get message payload
-    strcpy(payload, msg_get_payload(msg));
+    sng_strncpy(payload, msg_get_payload(msg), sizeof(payload));
 
     // Check how many lines we well need to draw this message
     payload_lines = 0;
@@ -189,7 +189,7 @@ call_raw_print_msg(ui_t *ui, sip_msg_t *msg)
 
     // Print msg header
     wattron(pad, A_BOLD);
-    mvwprintw(pad, info->padline++, 0, "%s", sip_get_msg_header(msg, header));
+    mvwprintw(pad, info->padline++, 0, "%s", sip_get_msg_header(msg, header, sizeof(header)));
     wattroff(pad, A_BOLD);
 
     // Print msg payload
