@@ -222,8 +222,7 @@ filter_handle_key(ui_t *ui, int key)
     // Get current field value.
     // We trim spaces with sscanf because and empty field is stored as
     // space characters
-    memset(field_value, 0, sizeof(field_value));
-    strcpy(field_value, field_buffer(current_field(info->form), 0));
+    sng_strncpy(field_value, field_buffer(current_field(info->form), 0), sizeof(field_value));
     strtrim(field_value);
 
     // Check actions for this key
@@ -352,7 +351,7 @@ filter_save_options(ui_t *ui)
         // We trim spaces with sscanf because and empty field is stored as
         // space characters
         memset(field_value, 0, sizeof(field_value));
-        strcpy(field_value, field_buffer(info->fields[field_id], 0));
+        sng_strncpy(field_value, field_buffer(info->fields[field_id], 0), sizeof(field_value));
         strtrim(field_value);
 
         // Set filter expression
@@ -387,9 +386,9 @@ filter_save_options(ui_t *ui)
             case FLD_FILTER_KDMQ:
                 if (!strcmp(field_value, "*")) {
                     if (strlen(method_expr)) {
-                        sprintf(method_expr + strlen(method_expr), ",%s", filter_field_method(field_id));
+                        snprintf(method_expr + strlen(method_expr), sizeof(method_expr) - strlen(method_expr), ",%s", filter_field_method(field_id));
                     } else {
-                        strcpy(method_expr, filter_field_method(field_id));
+                        sng_strncpy(method_expr, filter_field_method(field_id), sizeof(method_expr));
                     }
                 }
                 break;
@@ -461,7 +460,7 @@ filter_method_from_setting(const char *value)
     // If there's a method filter
     if (methods_len) {
         // Copy value into temporal array
-        strncpy(methods, value, sizeof(methods));
+        sng_strncpy(methods, value, sizeof(methods));
 
         // Replace all commas with pippes
         while ((comma = strchr(methods, ',')))
