@@ -356,6 +356,7 @@ draw_message_pos(WINDOW *win, sip_msg_t *msg, int starting)
     const char *cur_line, *payload, *method = NULL;
     int syntax = setting_enabled(SETTING_SYNTAX);
     const char *nonascii = setting_get_value(SETTING_CR_NON_ASCII);
+    int payload_len;
 
     // Default text format
     int attrs = A_NORMAL | COLOR_PAIR(CP_DEFAULT);
@@ -372,11 +373,12 @@ draw_message_pos(WINDOW *win, sip_msg_t *msg, int starting)
 
     // Get packet payload
     cur_line = payload = (const char *) msg_get_payload(msg);
+    payload_len=packet_payloadlen(msg->packet);
 
     // Print msg payload
     line = starting;
     column = 0;
-    for (i = 0; i < strlen(payload); i++) {
+    for (i = 0; i < payload_len; i++) {
         // If syntax highlighting is enabled
         if (syntax) {
             // First line highlight
@@ -663,8 +665,9 @@ dialog_confirm(const char *title, const char *text, const char *options)
     // Parse each line of payload looking for sdp information
     tofree = str = strdup((char*)options);
     i = 0;
-    while ((option = strsep(&str, ",")) != NULL) {
-        strcpy(opts[i++], option);
+    while (i < (sizeof(opts) / sizeof(opts[0])) && (option = strsep(&str, ",")) != NULL) {
+        sng_strncpy(opts[i], option, sizeof(opts[i]));
+        i++;
     }
     sng_free(tofree);
 
