@@ -79,6 +79,7 @@ usage()
            "    -F --no-config\t Do not read configuration from default config file\n"
            "    -T --text\t Save pcap to text file\n"
            "    -R --rotate\t\t Rotate calls when capture limit have been reached\n"
+           "    -T --telephone-event\t\t capture and parse RTP telephone-event packets\n"
 #ifdef USE_EEP
            "    -H --eep-send\t Homer sipcapture url (udp:X.X.X.X:XXXX)\n"
            "    -L --eep-listen\t Listen for encapsulated packets (udp:X.X.X.X:XXXX)\n"
@@ -164,6 +165,7 @@ main(int argc, char* argv[])
         { "config", required_argument, 0, 'f' },
         { "no-config", no_argument, 0, 'F' },
         { "text", required_argument, 0, 'T' },
+        { "telephone-event", no_argument, 0, 't' },
 #ifdef USE_EEP
         { "eep-listen", required_argument, 0, 'L' },
         { "eep-send", required_argument, 0, 'H' },
@@ -174,7 +176,7 @@ main(int argc, char* argv[])
 
     // Parse command line arguments that have high priority
     opterr = 0;
-    char *options = "hVd:I:O:B:pqtW:k:crl:ivNqDL:H:ERf:FT";
+    char *options = "hVd:I:O:B:pqtW:k:crl:ivNqDL:H:ERf:FT:t";
     while ((opt = getopt_long(argc, argv, options, long_options, &idx)) != -1) {
         switch (opt) {
             case 'h':
@@ -292,9 +294,11 @@ main(int argc, char* argv[])
                 rotate = 1;
                 setting_set_value(SETTING_CAPTURE_ROTATE, SETTING_ON);
                 break;
+            case 't':
+                setting_set_value(SETTING_TELEPHONE_EVENT, SETTING_ON);
+                break;
                 // Dark options for dummy ones
             case 'p':
-            case 't':
             case 'W':
                 break;
             case 'L':
@@ -418,19 +422,40 @@ main(int argc, char* argv[])
 
         // Try to build the bpf filter string with the rest
         memset(bpf, 0, sizeof(bpf));
-        for (i = optind; i < argc; i++)
-            sprintf(bpf + strlen(bpf), "%s ", argv[i]);
+        for (i = optind; i < argc; i++) {
+            size_t current_len = strlen(bpf);
+            size_t remaining_space = sizeof(bpf) - current_len;
+            if (remaining_space <= 1) {
+                fprintf(stderr, "BPF filter too long (max %zu characters)\n", sizeof(bpf) - 1);
+                return 1;
+            }
+            int written = snprintf(bpf + current_len, remaining_space, "%s ", argv[i]);
+            if (written < 0 || (size_t)written >= remaining_space) {
+                fprintf(stderr, "BPF filter too long (max %zu characters)\n", sizeof(bpf) - 1);
+                return 1;
+            }
+        }
 
         // Check if this BPF filter is valid
         if (capture_set_bpf_filter(bpf) != 0) {
             // BPF Filter invalid, check incluiding match_expr
             match_expr = 0;    // There is no need to parse all payload at this point
 
-
             // Build the bpf filter string
             memset(bpf, 0, sizeof(bpf));
-            for (i = optind - 1; i < argc; i++)
-                sprintf(bpf + strlen(bpf), "%s ", argv[i]);
+            for (i = optind - 1; i < argc; i++) {
+                size_t current_len = strlen(bpf);
+                size_t remaining_space = sizeof(bpf) - current_len;
+                if (remaining_space <= 1) {
+                    fprintf(stderr, "BPF filter too long (max %zu characters)\n", sizeof(bpf) - 1);
+                    return 1;
+                }
+                int written = snprintf(bpf + current_len, remaining_space, "%s ", argv[i]);
+                if (written < 0 || (size_t)written >= remaining_space) {
+                    fprintf(stderr, "BPF filter too long (max %zu characters)\n", sizeof(bpf) - 1);
+                    return 1;
+                }
+            }
 
             // Check bpf filter is valid again
             if (capture_set_bpf_filter(bpf) != 0) {
