@@ -62,7 +62,7 @@ media_destroyer(void *item)
 void
 media_set_type(sdp_media_t *media, const char *type)
 {
-    strncpy(media->type, type, MEDIATYPELEN - 1);
+    sng_strncpy(media->type, type, MEDIATYPELEN);
 }
 
 void
@@ -86,7 +86,7 @@ media_add_format(sdp_media_t *media, uint32_t code, const char *format)
         return;
 
     fmt->id = code;
-    strcpy(fmt->format, format);
+    sng_strncpy(fmt->format, format, sizeof(fmt->format));
     vector_append(media->formats, fmt);
 }
 
