@@ -147,6 +147,8 @@ sip_attr_color_method(const char *value)
             return COLOR_PAIR(CP_MAGENTA_ON_DEF);
         case SIP_METHOD_SUBSCRIBE:
             return COLOR_PAIR(CP_BLUE_ON_DEF);
+	case SIP_METHOD_KDMQ:
+	    return COLOR_PAIR(CP_CYAN_ON_DEF) | A_BOLD;
         default:
             return 0;
     }
